# -*- coding: utf-8 -*-
"""This module defines the `Ticker` class."""
import queue
import threading
import time
from typing import Optional


class Ticker:
    """Delivery of ticks at intervals.

    Once started the `Ticker` will periodically make a boolean "tick" of True
    available through the `tick` method. Uncollected ticks will not stack or queue
    up, and the Ticker will continue to tick regardless. When stopped `tick` will
    return False, and any uncollected tick will be lost.

    Example::

        ticker = Ticker()
        ticker.start(5)

        while ticker.tick():
            execute_task()

    """

    def __init__(self) -> None:
        self._lock = threading.Lock()
        self._tick: Optional[queue.LifoQueue] = None  # pylint: disable=E1136

    def _schedule(self, interval: int) -> None:
        time.sleep(interval)
        while True:
            self._lock.acquire()
            # There is a non-zero risk of _tick being set to None between entering the loop and acquiring the lock
            # thus it's better to perform the check here rather than as the while expression.
            if not self._tick:
                break
            # Only use one queue spot for ticking, the second spot is reserved for stopping.
            if self._tick.qsize() == 0:
                self._tick.put(True)
            self._lock.release()
            time.sleep(interval)

    def start(self, interval: int) -> None:
        """Start the ticker.

        Args:
            interval: Time between ticks.

        Raises:
            Exception if already running.

        """
        self._lock.acquire()
        if self._tick:
            raise RuntimeError("Ticker already started")
        self._tick = queue.LifoQueue(2)
        self._lock.release()
        thread = threading.Thread(target=self._schedule, args=(interval,), daemon=True)
        thread.start()

    def tick(self) -> bool:
        """Wait for a tick to be delivered.

        Will return immediately if ticker is stopped.

        Returns:
            True on tick, False if stopped.

        """
        if not self._tick:
            return False
        tick = self._tick.get()
        if not tick:
            self._lock.acquire()
            self._tick = None
            self._lock.release()
        return tick

    def stop(self) -> None:
        """Stop the ticker."""
        self._lock.acquire()
        if self._tick and self._tick.qsize() != 2:
            self._tick.put(False)
        self._lock.release()



# -*- coding: utf-8 -*-
"""``zsv.ticker`` enables flexible and idiomatic regular execution of tasks::

    from zsv.ticker import Ticker

    ticker = Ticker()
    ticker.start(5)

    while ticker.tick():
        execute_task()

``Ticker`` aims to be more idiomatic and easy to use than a time calculation and
sleep call, and further enables the instantaneous termination of a waiting
task::

    import signal
    from time import sleep
    from zsv.ticker import Ticker

    ticker = Ticker()
    ticker.start(5)

    def abort(signum, frame):
        ticker.stop()

    signal.signal(signal.SIGINT, abort)

    while ticker.tick():
        print("tick")
        sleep(2)
        print("tock")


The above script wraps a `stop` call in a signal handler registered to SIGINT:
hitting Ctrl+C after the script prints "tick" but before it prints "tock"
will yield a final "tock" before it terminates.

"""
from .ticker import Ticker

__all__ = ["Ticker"]

version_info = (1, 0, 2)
__version__ = "1.0.2"



#!/usr/bin/env python
# -*- coding: utf-8 -*-
import os
import re

from setuptools import find_namespace_packages, setup

NAME = "zsv.ticker"
here = os.path.abspath(os.path.dirname(__file__))

with open(os.path.join(here, "README.rst"), "r") as f:
    readme = f.read()

with open(os.path.join(here, "src", "zsv", "ticker", "__init__.py"), "r") as f:
    version = re.search('__version__ = "([^"]+)"', f.read()).group(1)

with open(os.path.join(here, "requirements.txt"), "r") as f:
    requires = [x.strip() for x in f if x.strip()]

with open(os.path.join(here, "requirements-test.txt"), "r") as f:
    requires_test = [x.strip() for x in f if x.strip()]

with open(os.path.join(here, "requirements-dev.txt"), "r") as f:
    requires_dev = [x.strip() for x in f if x.strip()]

setup(
    name=NAME,
    version=version,
    description="Enabling flexible and idiomatic regular execution of tasks.",
    entry_points=None,
    long_description=readme,
    long_description_content_type="text/x-rst",
    author="Tobias Dély",
    author_email="cleverhatcamouflage@gmail.com",
    python_requires=">=3.6",
    url="https://gitlab.com/tdely/zsv.ticker",
    namespace_packages=["zsv"],
    packages=find_namespace_packages(include=["zsv.*"], where="src"),
    package_dir={"": "src"},
    install_requires=requires,
    setup_requires=["pytest-runner"],
    tests_require=requires_test,
    extras_require={"test": requires_test, "dev": requires_dev},
    test_suite="tests",
    include_package_data=True,
    classifiers=[
        # Trove classifiers
        # Full list: https://pypi.python.org/pypi?%3Aaction=list_classifiers
        'Development Status :: 4 - Beta',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        "Programming Language :: Python",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.6",
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Topic :: Software Development :: Libraries',
        'Topic :: Software Development :: Libraries :: Python Modules',
    ],
)
